<?php

defined('_JEXEC') or die('Restricted access');

jimport('joomla.application.component.controller');

abstract class BaseController extends JController
{
    protected $_view   = null;
    protected $_db     = null;
    protected $_loader = null;
    
    protected $option    = ''; 
    protected $form_view = '';
    protected $buffer    = null;     
    
    public function __construct()
    {
        parent::__construct();
        
        $this->_db = JFactory::getDBO();
        $this->_loader = CLoader::getInstance();
        
        $this->option    = $this->_loader->component();
        $this->form_view = $this->_loader->getFormView();        
    }  
    
    public function display()
    {
        parent::display();
    }
    
    public function add()
    {        
        $edit      = JRequest::getBool('edit');
        $task      = JRequest::getVar('task', null, 'post');
                
        if (!$edit && $task) {            
            $link = JRoute::_('index.php?option=com_'. $this->option .'&view='. $this->form_view .'&task=add', false);
            $this->setRedirect($link);
        }
                
        JRequest::setVar('view', $this->form_view);
        JRequest::setVar('layout', 'form');   
            
        parent::display();        
    }    
    
    public function edit()
    {        
        list($id) = JRequest::getVar('cid', array(0), 'post'); 
        
        if ($id) {
            $link = JRoute::_('index.php?option=com_'. $this->option .'&view='. $this->form_view .'&task=edit&cid[]='. $id, false);        
            $this->setRedirect($link);
        }
        
        JRequest::setVar('edit', true);
        $this->add();        
    }     
    
    /**
     * Сохранение записи
     */
    public function save()
    {
        $this->beforesave();
        
        $this->_save();
        
        $this->aftersave();
    }        
    
    /**
     * Удаление записей
     * @public
     */
    public function remove()
    {
        $this->_remove();
    }    
    
    /**
     * Сохраняем установленный пользователем порядок
     * @public
     * @return  boolean
     */    
    public function saveorder()
    {
        $this->_saveorder();
    }    
    
    /**
     * Публикация записи
     * @public
     */    
    public function publish()
    {
        $this->_publish();
    }    
    /**
     * Снятие записи с публикации
     * @public
     */    
    public function unpublish()
    {
        $this->_unpublish();    
    }
    
    /**
     * Порядок выше
     * @public
     */
    public function orderup()
    {
        $this->_orderup();
    }    
    /**
     * Порядок ниже
     * @public
     */
    public function orderdown()
    {
        $this->_orderdown();
    }
    
    /**
     * Отмена действия (добавление, редактирование)
     * @public
     */
    public function cancel()
    {
        $this->_cancel();
    }    
    
    /** -------------------------------------------------------------- **/   
    
    /** 
     * 
     * --- Helpers Functions ---
     *   
     */            
    protected function _getTable($tbl = null)
    {                
        if($tbl == null) {
            $view = $this->_getView();
        } else {
            $view = $tbl;
        }
        
        $row = JTable::getInstance($view, 'Table');
        
        return $row;
    }
    
    protected function _getView()
    {
        if ($this->_view == null) {            
            $view = JRequest::getWord('view', null);
            
            if ($view == null) {
                $this->setError('Error view type');            
                return false;
            }        
            
            $this->_view = $view;
        }
        
        return $this->_view;
    }
    
    /** ------------------------------------------------------------------- **/
    
    public function beforesave()
    {
        return;
    }
        
    public function aftersave()
    {
        return;
    }  
    
    /**
     * Standart function for save
     * @protected
     */
    protected function _save($tbl = null)
    {
        JRequest::checkToken() or jexit('Invalid Token');
        
        $post  = JRequest::get('post');  
        
        if ($tbl == null) {            
            $view    = $this->_loader->getCurrent();
            $tbl     = $view;
        } 
        
        if ($this->_do_save($post, $tbl)) {
            $msg = JText::_('Row saved');
        }     
        
        $link = 'index.php?option=com_'. $this->option .'&view='. $view;        
        $this->setRedirect($link, $msg);        
    }
    
    private function _do_save($post, $view) 
    {        
        /* Определяем Id из массива */
        if ($post['cid']) {
            list($id) = $post['cid'];
            unset($post['cid']);
            $post['id'] = $id;                
        }   
        
        $row = $this->_getTable($view);                       
        
        if (!$row->bind($post)) {
 			JError::raise(2, 500, $row->getError());
			return false;
        }
        
        if (!$row->id && isset($row->ordering)) {
            $row->ordering = $row->getNextOrder();
        }
        
 		if (!$row->store()) {
			JError::raise(2, 500, $row->getError());
			return false;
		}   
        
        $this->buffer->id = $row->id;
        
        return true;        
    }

    /**
     * Standart function for cancel
     * @protected
     */
    protected function _cancel()
    {
        
        $view   = $this->_loader->getCurrent();
        
        $link = 'index.php?option=com_'. $this->option .'&view='. $view;
        $this->setRedirect($link, 'Редактирование отменено');
    }      
    
    /**
     * Standart functions for ordering
     * @protected
     */    
    protected function _saveorder()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );

        $cid   = JRequest::getVar('cid'  , array(), 'post', 'array');
        $order = JRequest::getVar('order', array(), 'post', 'array');

        JArrayHelper::toInteger($cid);
        JArrayHelper::toInteger($order); 
        
        $view = JRequest::getVar('view');        
        $row  = $this->_getTable($view);

        for ($i=0; $i < count($cid); $i++) {
            $row->load( (int) $cid[$i] );

            if ($row->ordering != $order[$i]) {
                $row->ordering = $order[$i];
                
                if (!$row->store()) {
                    $this->setError($this->_db->getErrorMsg());
                    return false;
                }
            }
        }

        $row->reorder();
        
        $msg    = 'Порядок успешно изменен';
        $this->setRedirect('index.php?option=com_'. $this->option .'&view='. $view, $msg);
        
        return true;  
    } 
      
    protected function _orderdown()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );

        $this->_do_order(1);
        
        $view   = $this->_getView();
        $this->setRedirect('index.php?option=com_'. $this->option .'&view='. $view);        
    }
    
    protected function _orderup()
    {
        JRequest::checkToken() or jexit( 'Invalid Token' );

        $this->_do_order(-1);
        
        $view   = $this->_getView();
        $this->setRedirect('index.php?option=com_'. $this->option .'&view='. $view);
    }   
     
    private function _do_order($i)
    {
        list($id) = JRequest::getVar('cid', array(0), '', 'array');
        
        $row = $this->_getTable();
        
        $row->load( (int) $id );
        $row->move($i); 
        $row->reorder();
        
        return true;       
    }
    
    
    /**
     * Standart functions for published
     * @protected
     */     
    protected function _unpublish()
    {
        JRequest::checkToken() or jexit('Invalid Token');

        $cid = JRequest::getVar('cid', array(), 'post', 'array');
        JArrayHelper::toInteger($cid);

        if (count( $cid ) < 1) {
            JError::raiseError(500, 'Выберите записи для публикации');
        }
        
        if (!$this->_do_publish($cid, 0)) {
            echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
        }
        
        $view   = $this->_getView();
        
        $msg = 'Снято с публикации';
        
        $this->setRedirect('index.php?option=com_'. $this->option .'&view='. $view, $msg);            
    }  
      
    protected function _publish()
    {
        JRequest::checkToken() or jexit('Invalid Token');

        $cid = JRequest::getVar('cid', array(), 'post', 'array');
        JArrayHelper::toInteger($cid);

        if (count( $cid ) < 1) {
            JError::raiseError(500, 'Выберите записи для публикации');
        }
        
        if (!$this->_do_publish($cid)) {
            echo "<script> alert('".$model->getError(true)."'); window.history.go(-1); </script>\n";
        }
        
        $view   = $this->_getView();
        $msg = 'Опубликовано';
        
        $this->setRedirect('index.php?option=com_'. $this->option .'&view='. $view, $msg);        
    }
    
    private function _do_publish($cid = array(), $publish = 1)
    {
        $row = $this->_getTable();

        if (!$row->publish($cid, $publish)) {
        	$this->setError($this->_db->getErrorMsg());
        	return false;
        }      
        
        return true;  
    } 
    
    
    /**
     * Standart function for remove
     * @protected
     */        
    protected function _remove()
    {
        $post = count(JRequest::getVar('cid', array(), 'post', 'array'));
        
        if ($post) {
            JRequest::checkToken() or jexit('Invalid Token');    
        }        
        
        $cid = JRequest::getVar( 'cid', array(), null, 'array' );
        JArrayHelper::toInteger($cid);

        if (count( $cid ) < 1) {
            JError::raiseWarning(500, JText::_('Select an item to delete'));
            return false;            
        }        
        
        if (!$this->_do_remove($cid)) {
            $msg = JText::_('Error delete items');
        } else {
            $msg = JText::sprintf( JText::_('Removed'), count($cid) );
        }
        
        $view   = $this->_getView();
        $option = $this->_loader->component();
        
        $this->setRedirect('index.php?option=com_'. $this->option .'&view='. $view, $msg);        
    }  
    private function _do_remove($cid = array())
    {
        $cids = implode(',', $cid);
        
        if ($this->canDelete($cids)) {            
            $view = $this->_getView();
            
            $query = 'DELETE FROM #__'. $this->option .'_'. $view . ' WHERE id IN ('. $cids .')';
            $this->_db->setQuery($query); 
            $this->_db->query();
            
            return true;       
        } else {
            JError::raiseNotice(500, JText::_('Item is use'));            
            return false;
        }
    }
      
    public function canDelete($cids = '')
    {
        return true;
    }        
}